@@ -25,11 +25,15 @@
 
 # AUTHORS
 # Hervé BREDIN - http://herve.niderb.fr
+# Hadrien TITEUX - https://github.com/hadware
 
 
 from typing import Iterable, Any
 from optuna.trial import Trial
 
+from .pipeline import Pipeline
+from collections.abc import Mapping
+
 
 class Parameter:
     """Base hyper-parameter"""
@@ -60,16 +64,16 @@ class DiscreteUniform(Parameter):
     """Discrete uniform hyper-parameter
 
     The value is sampled from the range [low, high],
-    and the step of discretization is `q`.
+    and the step of discretization is `q`.
 
     Parameters
     ----------
-    low : `float`
+    low : `float`
         Lower endpoint of the range of suggested values.
-        `low` is included in the range.
+        `low` is included in the range.
     high : `float`
         Upper endpoint of the range of suggested values.
-        `high` is included in the range.
+        `high` is included in the range.
     q : `float`
         A step of discretization.
     """
@@ -91,12 +95,12 @@ class Integer(Parameter):
 
     Parameters
     ----------
-    low : `int`
+    low : `int`
         Lower endpoint of the range of suggested values.
-        `low` is included in the range.
+        `low` is included in the range.
     high : `int`
         Upper endpoint of the range of suggested values.
-        `high` is included in the range.
+        `high` is included in the range.
     """
 
     def __init__(self, low: int, high: int):
@@ -115,12 +119,12 @@ class LogUniform(Parameter):
 
     Parameters
     ----------
-    low : `float`
+    low : `float`
         Lower endpoint of the range of suggested values.
-        `low` is included in the range.
+        `low` is included in the range.
     high : `float`
         Upper endpoint of the range of suggested values.
-        `high` is excluded from the range.
+        `high` is excluded from the range.
     """
 
     def __init__(self, low: float, high: float):
@@ -139,12 +143,12 @@ class Uniform(Parameter):
 
     Parameters
     ----------
-    low : `float`
+    low : `float`
         Lower endpoint of the range of suggested values.
-        `low` is included in the range.
+        `low` is included in the range.
     high : `float`
         Upper endpoint of the range of suggested values.
-        `high` is excluded from the range.
+        `high` is excluded from the range.
     """
 
     def __init__(self, low: float, high: float):
@@ -173,3 +177,28 @@ class Frozen(Parameter):
 
     def __call__(self, name: str, trial: Trial):
         return self.value
+
+
+class ParamDict(Pipeline, Mapping):
+    """Dict-like structured hyper-parameter
+
+    Usage
+    -----
+    >>> params = ParamDict(param1=Uniform(0.0, 1.0), param2=Uniform(-1.0, 1.0))
+    >>> params = ParamDict(**{"param1": Uniform(0.0, 1.0), "param2": Uniform(-1.0, 1.0)})
+    """
+
+    def __init__(self, **params):
+        super().__init__()
+        self.__params = params
+        for param_name, param_value in params.items():
+            setattr(self, param_name, param_value)
+
+    def __len__(self):
+        return len(self.__params)
+
+    def __iter__(self):
+        return iter(self.__params)
+
+    def __getitem__(self, param_name):
+        return getattr(self, param_name)
